import json

from django.contrib.postgres.fields import JSONField
from django.contrib.postgres.forms import JSONField as JSONFormField


class SRPatchedJSONFormField(JSONFormField):
    """
    Overrides `prepare_value` to return value as it is without dumping to json, and thus
      re-encoding over and over again
    """

    def prepare_value(self, value):
        """
        Exclude returning None in case of {}, returning None makes the field empty when loaded
        from admin instead of returning {},
        which causes an error where empty dict was expected
        """
        if not value == {} and value in self.empty_values:
            return None
        elif isinstance(value, str):
            return value
        else:
            return super(SRPatchedJSONFormField, self).prepare_value(value)


class SRPatchedJSONField(JSONField):
    """
    Overrides
    1. `formfield` to set our custom JSON field as `form_class`
    2. `from_db_value` to return a json from `str`
    """

    def formfield(self, **kwargs):
        defaults = {"form_class": SRPatchedJSONFormField}
        defaults.update(kwargs)
        return super(JSONField, self).formfield(**defaults)

    def from_db_value(self, value, expression, connection, context):
        """
        To solve https://bitbucket.org/schinckel/django-jsonfield/issues/57/cannot-use-in-the-same-project-as-djangos
        """
        if isinstance(value, str):
            try:
                return json.loads(value)
            except ValueError:
                return value

        return value



# -*- coding: utf-8 -*-
from django.db.models.fields import related
from .compat import get_field, get_remote_field_to


class InvalidConstraint(Exception):
    def __init__(self, fields, *args, **kwargs):
        self.fields = fields
        super(InvalidConstraint, self).__init__(*args, **kwargs)


def _is_unique_field(field):
    if not field.unique:
        return False
    if field.primary_key:
        # Primary key fields should not generally be checked for unique constraints, except when the
        # primary key is a OneToOne mapping to an external table not via table inheritance, in which
        # case we don't want to create new objects which will overwrite existing objects.
        return (isinstance(field, related.OneToOneField) and
                not issubclass(field.model, get_remote_field_to(field)))
    else:
        return True


def unique_constraint(model, instance):
    error_fields = []
    for field in instance._meta.fields:
        if _is_unique_field(field):
            value = getattr(instance, field.name)

            # If the value is none and the field allows nulls, skip it
            if value is None and field.null:
                continue

            check = {field.name: value}

            if model._default_manager.filter(**check).exists():
                error_fields.append(field)
    if error_fields:
        raise InvalidConstraint(error_fields)


def unique_together_constraint(model, instance):
    if not instance._meta.unique_together:
        return
    error_fields = []
    for unique_fields in instance._meta.unique_together:
        check = {}
        for field_name in unique_fields:
            if not get_field(instance, field_name).primary_key:
                check[field_name] = getattr(instance, field_name)
        if all(e is None for e in check.values()):
            continue

        if model._default_manager.filter(**check).exists():
            error_fields.extend([
                get_field(instance, field_name)
                for field_name in unique_fields
            ])
    if error_fields:
        raise InvalidConstraint(error_fields)



# coding=utf-8

import io
from PIL import Image
from PIL import ImageDraw
from PIL import ImageColor
from PIL import ImageFont
from PIL import ImageOps

get_color = lambda name: ImageColor.getrgb(name)


def get_placeholder_image(width, height, name=None, fg_color=get_color('black'),
        bg_color=get_color('grey'), text=None, font=u'Verdana.ttf',
        fontsize=42, encoding=u'unic', mode='RGBA', fmt=u'PNG'):
    """Little spin-off from https://github.com/Visgean/python-placeholder
    that not saves an image and instead returns it."""
    size = (width, height)
    text = text if text else '{0}x{1}'.format(width, height)

    try:
        font = ImageFont.truetype(font, size=fontsize, encoding=encoding)
    except IOError:
        font = ImageFont.load_default()

    result_img = Image.new(mode, size, bg_color)

    text_size = font.getsize(text)
    text_img = Image.new("RGBA", size, bg_color)

    #position for the text:
    left = size[0] / 2 - text_size[0] / 2
    top = size[1] / 2 - text_size[1] / 2

    drawing = ImageDraw.Draw(text_img)
    drawing.text((left, top),
                 text,
                 font=font,
                 fill=fg_color)

    txt_img = ImageOps.fit(text_img, size, method=Image.BICUBIC, centering=(0.5, 0.5))

    result_img.paste(txt_img)
    file_obj = io.BytesIO()
    txt_img.save(file_obj, fmt)

    return file_obj.getvalue()


import django


def get_GenericForeignKey():
    try:
        from django.contrib.contenttypes.fields import GenericForeignKey
    # For Django 1.6 and earlier
    except ImportError:
        from django.contrib.contenttypes.generic import GenericForeignKey
    return GenericForeignKey


def get_GenericRelation():
    try:
        from django.contrib.contenttypes.fields import GenericRelation
    # For Django 1.6 and earlier
    except ImportError:
        from django.contrib.contenttypes.generic import GenericRelation
    return GenericRelation

try:
    from collections import OrderedDict
except ImportError:
    from django.utils.datastructures import SortedDict as OrderedDict


try:
    import importlib
except ImportError:
    from django.utils import importlib


try:
    from django.db.transaction import atomic
# For django 1.5 and earlier
except ImportError:
    from django.db.transaction import commit_on_success as atomic


try:
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models import get_model


def get_field(model, field_name):
    if django.VERSION < (1, 8):
        return model._meta.get_field_by_name(field_name)[0]
    else:
        return model._meta.get_field(field_name)


def get_remote_field(field):
    if django.VERSION < (1, 9):
        return field.rel
    else:
        return field.remote_field


def get_remote_field_to(field):
    if django.VERSION < (1, 9):
        return field.rel.to
    else:
        return field.remote_field.model


try:
    # added in Python 3.0
    from inspect import signature
    def getargnames(callable):
        return list(signature(callable).parameters.keys())

except ImportError:
    # loud DeprecationWarnings in 3.5
    from inspect import getargspec
    def getargnames(callable):
        return getargspec(callable).args
